"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const jira_client_1 = __importDefault(require("jira-client"));
const tempo_client_1 = __importDefault(require("tempo-client"));
const jira_issue_1 = __importDefault(require("./entity/jira-issue"));
const worklog_1 = __importDefault(require("./entity/worklog"));
const summary_item_1 = __importDefault(require("./entity/summary-item"));
const generate_email_1 = __importDefault(require("./generate-email"));
const user_1 = __importDefault(require("./entity/user"));
class TempoSummaryEmail {
    constructor(options) {
        this.tempoApiKey = options.tempoApiKey;
        this.jiraUsername = options.jiraUsername;
        this.jiraApiKey = options.jiraApiKey;
        this.jiraDomain = options.jiraDomain;
    }
    generateEmailForRange(fromDate, toDate) {
        return __awaiter(this, void 0, void 0, function* () {
            const summaryItems = yield this.retrieveSummaryItems(fromDate, toDate);
            const showDates = (fromDate !== toDate);
            return (new generate_email_1.default(yield this.getUser(), summaryItems, showDates)).generateEmail();
        });
    }
    retrieveSummaryItems(from, to) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield this.getUser();
            const tempo = this.createTempoClient();
            const jira = this.createJiraClient();
            const tempoWorklogs = yield tempo.worklogs.getForUser(user.getAccountId(), { from, to }).then((response) => { return response.results; });
            const summaryItems = {};
            for (const index in tempoWorklogs) {
                if (!Object.prototype.hasOwnProperty.call(tempoWorklogs, index)) {
                    continue;
                }
                const worklog = new worklog_1.default(tempoWorklogs[index]);
                let summaryItem;
                if (worklog.getIssueKey() in summaryItems) {
                    summaryItem = summaryItems[worklog.getIssueKey()];
                }
                else {
                    const issue = yield jira.findIssue(worklog.getIssueKey());
                    const jiraIssue = new jira_issue_1.default(issue, this.jiraDomain);
                    summaryItem = new summary_item_1.default(jiraIssue);
                    // eslint-disable-next-line require-atomic-updates
                    summaryItems[worklog.getIssueKey()] = summaryItem;
                }
                summaryItem.addWorklog(worklog);
            }
            return Object.keys(summaryItems).map(i => summaryItems[i]);
        });
    }
    createTempoClient() {
        return new tempo_client_1.default({
            apiVersion: "3",
            bearerToken: this.tempoApiKey,
            host: "api.tempo.io",
            protocol: "https",
        });
    }
    createJiraClient() {
        return new jira_client_1.default({
            apiVersion: "3",
            host: this.jiraDomain,
            password: this.jiraApiKey,
            protocol: "https",
            strictSSL: true,
            username: this.jiraUsername,
        });
    }
    getUser() {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof (this.user) !== "undefined") {
                return this.user;
            }
            const jira = this.createJiraClient();
            const response = yield jira.getCurrentUser().catch((reason) => {
                throw new Error(reason);
            });
            this.user = new user_1.default(response);
            return this.user;
        });
    }
}
exports.default = TempoSummaryEmail;
//# sourceMappingURL=tempo-summary-email.js.map